<!DOCTYPE html>
<html>

<head>
    <meta charset="utf-8" />
    <title>Verify your ActivityPub Actor</title>
</head>

<body>
    <h1>Verify your ActivityPub Actor</h1>

    <form action="/verify" method="POST">
        <input type="text" name="actor_uri" />
        <input type="submit" value="Verify" />

    </form>

</body>

</html>